Package: DSAIDE
Type: Package
Title: Dynamical Systems Approach to Infectious Disease Epidemiology
        (Ecology/Evolution)
Description: Exploration of simulation models (apps) of various infectious disease transmission dynamics scenarios.
    The purpose of the package is to help individuals learn 
    about infectious disease epidemiology (ecology/evolution) from a dynamical systems perspective.
    All apps include explanations of the underlying models and instructions on what to do with the models. 
Version: 1.0.0
Date: 2026-07-27
Authors@R: c(
    person("Andreas", "Handel", email = "andreas.handel@gmail.com", role = c("aut", "cre"), comment = c(ORCID = "0000-0002-4622-1146") ),
    person("Cody", "Dailey",  role = "ctb"), 
	person("Isaac Chun-Hai", "Fung",  role = "ctb"), 
    person("Ronald", "Galiwango",  role = "ctb"), 
    person("Salil", "Goyal",  role = "ctb"),
	person("Spencer", "Hall",  role = "ctb"), 
	person("Ben", "Listyg",  role = "ctb"), 
    person("Brian", "McKay",  role = "ctb"),
    person("John", "Rossow",  role = "ctb"),
    person("Sina", "Solaimanpour",  role = "ctb"),
    person("Alexis", "Vittengl",  role = "ctb"),
    person("Henok", "Woldu",  role = "ctb")
    )
License: GPL-3
Encoding: UTF-8
LazyData: TRUE
Imports: adaptivetau, deSolve, dplyr, ggplot2, gridExtra, lhs, nloptr,
        plotly, rlang, stats, utils, XML
Depends: R (>= 4.0.0), shiny (>= 1.2)
Suggests: covr, devtools, emoji, knitr, pkgdown, rmarkdown, roxygen2,
        testthat
VignetteBuilder: knitr
URL: https://andreashandel.github.io/DSAIDE/
BugReports: https://github.com/andreashandel/DSAIDE/issues
Config/roxygen2/version: 8.0.0
NeedsCompilation: no
Packaged: 2026-07-27 20:16:15 UTC; Andreas
Author: Andreas Handel [aut, cre] (ORCID:
    <https://orcid.org/0000-0002-4622-1146>),
  Cody Dailey [ctb],
  Isaac Chun-Hai Fung [ctb],
  Ronald Galiwango [ctb],
  Salil Goyal [ctb],
  Spencer Hall [ctb],
  Ben Listyg [ctb],
  Brian McKay [ctb],
  John Rossow [ctb],
  Sina Solaimanpour [ctb],
  Alexis Vittengl [ctb],
  Henok Woldu [ctb]
Maintainer: Andreas Handel <andreas.handel@gmail.com>
Repository: CRAN
Date/Publication: 2026-07-27 20:50:02 UTC
Built: R 4.5.2; ; 2026-07-27 21:12:05 UTC; unix
