# Generated by roxygen2: do not edit by hand

export(dsaidemenu)
export(generate_documentation)
export(generate_fctcall)
export(generate_ggplot)
export(generate_plotly)
export(generate_shinyinput)
export(generate_text)
export(run_model)
export(simulate_Characteristics_of_ID_ode)
export(simulate_Complex_ID_Control_ode)
export(simulate_Drug_Resistance_Evolution_stochastic)
export(simulate_Environmental_Transmission_model_ode)
export(simulate_Host_Heterogeneity_Model_ode)
export(simulate_SEIRSd_model_stochastic)
export(simulate_SIRSd_model_ode)
export(simulate_SIRSd_model_stochastic)
export(simulate_SIR_model_discrete)
export(simulate_SIR_model_ode)
export(simulate_SIR_modelexploration)
export(simulate_SIR_usanalysis)
export(simulate_Vector_transmission_model_ode)
export(simulate_directtransmission_ode)
export(simulate_flu_fit)
export(simulate_idcontrolmultigroup_ode)
export(simulate_idcontrolmultioutbreak_ode)
export(simulate_idpatterns_ode)
export(simulate_idsurveillance_ode)
export(simulate_idvaccine_ode)
export(simulate_multipathogen_ode)
export(simulate_noro_fit)
import(ggplot2, except = last_plot)
import(plotly)
import(shiny)
importFrom(gridExtra,grid.arrange)
importFrom(nloptr,nloptr)
importFrom(rlang,.data)
importFrom(stats,median)
importFrom(stats,reshape)
importFrom(utils,head)
importFrom(utils,tail)
